"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.activate = activate;
exports.deactivate = deactivate;
const vscode = require("vscode");
const yaml = require("js-yaml");
const fs = require("fs");
const path = require("path");
const os = require("os");
function activate(context) {
    ///// Add NewFile
    context.subscriptions.push(vscode.commands.registerCommand('sp.createNewFile', async () => {
        const now = new Date();
        const timestamp = now.getTime();
        const uri = vscode.Uri.parse(`untitled:新しいシナリオ-${timestamp}.sp.yml`);
        const document = await vscode.workspace.openTextDocument(uri);
        const editor = await vscode.window.showTextDocument(document);
        editor.edit(editBuilder => {
            editBuilder.insert(new vscode.Position(0, 0), `title: "新しいシナリオ"

sequences:
  - id: 1
    name: "新しいシーケンス"
    description: "ここに説明を書きます"
    scenes:
      - id: 1
        name: "最初のシーン"
        duration: null
        time_of_day: "朝"
        se: null
        description: ""
        actors: []
        cuts:
          - id: 1
            camera: ""
            description: ""
            se: null
            dialogue: []
            duration: 3
            transition: null
`);
        });
    }));
    ///// Add Template
    context.subscriptions.push(vscode.commands.registerCommand('sp.addTemplate', async () => {
        const editor = vscode.window.activeTextEditor;
        if (!editor) {
            vscode.window.showErrorMessage('エディターが開かれていません。');
            return;
        }
        const template = `
title: "タイトルをここに"
sequences:
  - id: 1
    name: "オープニング"
    description: "ストーリー導入部"
    scenes:
      - id: 1
        name: "最初のシーン"
        description: "シーンの説明"
        cuts:
          - id: 1
            camera: "カメラの説明"
            description: "カット内容"
            se: null
            dialogue: []
            duration: 3
            transition: null
`;
        const position = editor.selection.active;
        await editor.edit(editBuilder => {
            editBuilder.insert(position, template);
        });
    }));
    ///// Add Sequences
    context.subscriptions.push(vscode.commands.registerCommand('sp.addSequences', async () => {
        const editor = vscode.window.activeTextEditor;
        if (!editor) {
            vscode.window.showErrorMessage('エディターが開かれていません。');
            return;
        }
        const position = editor.selection.active;
        const lineText = editor.document.lineAt(position.line).text;
        const baseIndent = ' '.repeat(2); // 2スペース
        const snippet = `
${baseIndent}- id: 
${baseIndent}  name: ""
${baseIndent}  description: ""
${baseIndent}  scenes: []
  `;
        await editor.edit(editBuilder => {
            editBuilder.insert(position, snippet);
        });
    }));
    ///// Add Scene
    context.subscriptions.push(vscode.commands.registerCommand('sp.addScene', async () => {
        const editor = vscode.window.activeTextEditor;
        if (!editor) {
            vscode.window.showErrorMessage('アクティブなドキュメントがありません');
            return;
        }
        const position = editor.selection.active;
        const newPosition = position.with(position.line, 0);
        editor.selection = new vscode.Selection(newPosition, newPosition);
        const baseIndent = ' '.repeat(6); // 6スペース
        const snippet = `
${baseIndent}- id: 
${baseIndent}  name: "新しいシーン"
${baseIndent}  duration: null
${baseIndent}  time_of_day: "昼"
${baseIndent}  se: null
${baseIndent}  description: ""
${baseIndent}  actors: []
${baseIndent}  cuts:
${baseIndent}    - id: 
${baseIndent}      camera: ""
${baseIndent}      description: ""
${baseIndent}      se: null
${baseIndent}      dialogue: []
${baseIndent}      duration: 3
${baseIndent}      transition: null
`;
        editor.insertSnippet(new vscode.SnippetString(snippet), editor.selection.active);
    }));
    ///// Add Cut
    context.subscriptions.push(vscode.commands.registerCommand('sp.addCut', async () => {
        const editor = vscode.window.activeTextEditor;
        if (!editor) {
            vscode.window.showErrorMessage('アクティブなドキュメントがありません');
            return;
        }
        const position = editor.selection.active;
        const newPosition = position.with(position.line, 0);
        editor.selection = new vscode.Selection(newPosition, newPosition);
        // 固定インデントでスニペット作成（例：10スペース）
        const baseIndent = ' '.repeat(10);
        const snippet = `
${baseIndent}- id: 
${baseIndent}  camera: ""
${baseIndent}  description: ""
${baseIndent}  se: null
${baseIndent}  dialogue: []
${baseIndent}  duration: 3
${baseIndent}  transition: null
`;
        editor.insertSnippet(new vscode.SnippetString(snippet), newPosition);
    }));
    ///// Export Html&CSS
    context.subscriptions.push(vscode.commands.registerCommand('sp.exportHtml', async () => {
        const editor = vscode.window.activeTextEditor;
        if (!editor) {
            vscode.window.showErrorMessage('出力対象が選択されていません');
            return;
        }
        const document = editor.document;
        const text = document.getText();
        // YAML読み込み & 型アサーション
        let yamlContent;
        try {
            yamlContent = yaml.load(text);
        }
        catch (e) {
            vscode.window.showErrorMessage('YAMLのパースエラー: ' + e.message);
            return;
        }
        if (!yamlContent) {
            vscode.window.showErrorMessage('YAMLが空です。');
            return;
        }
        // レイアウト選択
        const layout = await vscode.window.showQuickPick(["縦書き（脚本風）", "横書き（モダン）", "印刷用（白黒）"], { placeHolder: "レイアウトスタイルを選択してください" });
        if (!layout) {
            vscode.window.showInformationMessage("出力がキャンセルされました。");
            return;
        }
        let cssFileName = "style-vertical.css";
        if (layout.includes("横書き"))
            cssFileName = "style-horizontal.css";
        if (layout.includes("印刷用"))
            cssFileName = "style-print.css";
        // ファイル名生成（安全なタイトル変換）
        const safeTitle = (yamlContent.title || "scenario")
            .replace(/\s+/g, "_")
            .replace(/[\\/:*?"<>|]/g, "_");
        const defaultFileName = `${safeTitle}.html`;
        const workspaceFolder = vscode.workspace.workspaceFolders?.[0]?.uri.fsPath || os.homedir();
        const uri = await vscode.window.showSaveDialog({
            defaultUri: vscode.Uri.file(path.join(workspaceFolder, defaultFileName)),
            filters: { 'HTML Files': ['html'] }
        });
        if (!uri) {
            vscode.window.showInformationMessage('書き出しがキャンセルされました。');
            return;
        }
        const styleUri = vscode.Uri.file(path.join(path.dirname(uri.fsPath), 'style.css'));
        const html = generateHtml(yamlContent, styleUri);
        fs.writeFileSync(uri.fsPath, html, 'utf8');
        const extensionPath = context.extensionPath;
        const cssSourcePath = path.join(extensionPath, 'media', cssFileName);
        const cssTargetPath = path.join(path.dirname(uri.fsPath), 'style.css');
        fs.copyFile(cssSourcePath, cssTargetPath, async (err) => {
            if (err) {
                vscode.window.showErrorMessage('CSSコピーに失敗しました。');
                return;
            }
            vscode.window.showInformationMessage(`HTMLとCSS（${layout}）を書き出しました。`);
            await vscode.env.openExternal(uri);
        });
    }));
    ///// Preview
    context.subscriptions.push(vscode.commands.registerCommand('sp.previewScene', () => {
        previewScene(context);
    }));
    vscode.window.showInformationMessage('ScreenPlay Formater 拡張機能が有効になりました！');
}
function deactivate() { }
function previewScene(context) {
    const editor = vscode.window.activeTextEditor;
    if (!editor) {
        vscode.window.showErrorMessage("アクティブなドキュメントがありません");
        return;
    }
    const text = editor.document.getText();
    let parsed;
    try {
        parsed = yaml.load(text);
    }
    catch (err) {
        vscode.window.showErrorMessage("YAMLパースに失敗しました");
        return;
    }
    const panel = vscode.window.createWebviewPanel("scenePreview", "シナリオプレビュー", vscode.ViewColumn.Beside, {});
    // @ts-ignore
    const styleUri = panel.webview.asWebviewUri(
    // @ts-ignore
    vscode.Uri.joinPath(context.extensionUri, 'media', 'style.css'));
    const html = generateHtml(parsed, styleUri);
    panel.webview.html = html;
}
///// HTML生成関数
function generateHtml(data, styleUri) {
    if (!data || !data.sequences) {
        return "<h2>シナリオが正しく構造化されていません</h2>";
    }
    let html = `
  <html>
  <head>
    <meta charset="UTF-8" />
    <link href="${styleUri}" rel="stylesheet" />
  </head>
  <body>
  `;
    html += `<div class="scenario-title">${data.title || "（タイトル未設定）"}</div>`;
    for (const seq of data.sequences) {
        html += `<div class="sequence" data-id="${seq.id}">
      <div class="sequence-header">シーケンス #${seq.id}: ${seq.name}</div>
      <div class="sequence-description">${seq.description || ""}</div>`;
        for (const scene of seq.scenes || []) {
            html += `<div class="scene" data-id="${scene.id}">
        <div class="scene-header">シーン #${scene.id}: ${scene.name}</div>
        <div class="scene-time">${scene.time_of_day || "時間未設定"}</div>`;
            for (const cut of scene.cuts || []) {
                html += `<div class="cut" data-id="${cut.id}">
          <div class="cut-header">カット #${cut.id}</div>
          <div class="camera">カメラ: ${cut.camera || ""}</div>
          <div class="cut-description">描写: ${cut.description || ""}</div>
          <div class="cut-duration">時間: ${cut.duration || "?"}秒</div>
          <div class="dialogue">セリフ:`;
                for (const d of cut.dialogue || []) {
                    html += `<div><span class="speaker">${d.speaker}:</span> ${d.text} (${d.description || ""})</div>`;
                }
                html += `</div>
          <div class="transition">トランジション: ${cut.transition || ""}</div>
        </div>`;
            }
            html += `</div>`; // scene
        }
        html += `</div>`; // sequence
    }
    html += "</body></html>";
    return html;
}
/*
function generateExportHtml(data: any): string {
  if (!data || !data.sequences) {
    return "<h2>シナリオが正しく構造化されていません</h2>";
  }

  const sceneFields = {
    name: "シーン名",
    time_of_day: "時間帯",
    description: "説明"
  };

  const cutFields = {
    camera: "カメラ",
    description: "描写",
    duration: "時間（秒）",
    transition: "トランジション",
    se: "SE"
  };

  const htmlParts: string[] = [];

  htmlParts.push(`
<!DOCTYPE html>
<html lang="ja">
<head>
  <meta charset="UTF-8" />
  <title>${data.title || "タイトル未設定"}</title>
  <link rel="stylesheet" href="style.css" />
</head>
<body>
  <div class="scenario-title">${data.title || "（タイトル未設定）"}</div>
  `);

  for (const seq of data.sequences) {
    htmlParts.push(`<div class="sequence">
      <div class="sequence-name">${seq.name}</div>
      <div class="sequence-description">${seq.description || ""}</div>`);

    for (const scene of seq.scenes || []) {
      htmlParts.push(renderBlock(scene, sceneFields, "scene"));

      for (const cut of scene.cuts || []) {
        htmlParts.push(renderBlock(cut, cutFields, "cut"));

        // セリフは個別処理（多階層）

        if (cut.dialogue && Array.isArray(cut.dialogue)) {
          const dialogueHtml = (cut.dialogue as DialogueLine[]).map((d: DialogueLine) =>
            `<div class="dialogue-line"><span class="speaker">${d.speaker || ""}:</span> ${d.text || ""} (${d.description || ""})</div>`
          ).join("");
          htmlParts.push(`<div class="dialogue">セリフ: ${dialogueHtml}</div>`);
        }
      }
    }

    htmlParts.push(`</div>`); // end of sequence
  }

  htmlParts.push("</body></html>");
  return htmlParts.join("\n");
}
  */
function renderBlock(data, fields, className) {
    const parts = [`<div class="${className}">`];
    for (const key in fields) {
        parts.push(`<div class="${key}">${fields[key]}: ${data[key] ?? ""}</div>`);
    }
    parts.push(`</div>`);
    return parts.join("\n");
}
//# sourceMappingURL=extension%20-%20%E3%82%B3%E3%83%94%E3%83%BC.js.map